/* functions -- code for implementing annotation measurement in SML */

#if defined(WIN32)
#include <windows.h>
#endif

#include "SFSCONFG.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "sml.h"


/* protected versions of maths functions */
double Sqrt()
{
	double	d;
	d=pop();
	if ((d==ERRVAL) || (d < 0.0)) {
		strcpy(lasterr,"square root of error/negative number");
		return(ERRVAL);
	}
	else
		return(sqrt(d));
}
double Abs()
{
	double	d,fabs();
	d=pop();
	if (d==ERRVAL)
		return(ERRVAL);
	else
		return(fabs(d));
}
double Int()
{
	double	d;
	int	i;
	d=pop();
	if (d==ERRVAL)
		return(ERRVAL);
	else {
		i=(int)d;
		return((double)i);
	}
}
double Log()
{
	double	d;
	d=pop();
	if ((d==ERRVAL) || (d <= 0.0)) {
		strcpy(lasterr,"logarithm of zero/negative number");
		return(ERRVAL);
	}
	else
		return(log(d));
}
double Log10()
{
	double	d;
	d=pop();
	if ((d==ERRVAL) || (d <= 0.0)) {
		strcpy(lasterr,"logarithm of zero/negative number");
		return(ERRVAL);
	}
	else
		return(log10(d));
}
double Exp()
{
	double	d;
	d=pop();
	if ((d==ERRVAL) || (d > 1000.0)) {
		strcpy(lasterr,"exponential of error/too large value");
		return(ERRVAL);
	}
	else
		return(exp(d));
}
double Sin() { return sin(pop()); }
double Cos() { return cos(pop()); }
double Tan() { return tan(pop()); }
double Asin() { return asin(pop()); }
double Acos() { return acos(pop()); }
double Atan() { return atan(pop()); }
double Atan2()
{
	double n,d;
	d=pop();
	n=pop();
	return atan2(n,d);
}
double Pow()
{
	double b,e;
	e=pop();
	b=pop();
	return pow(b,e);
}

#ifdef DOS
long times(buff)
long	*buff;
{
	buff[0]=time((time_t *)0);
	buff[1]=buff[2]=buff[2]=0;
	return(buff[0]);
}
#endif

/* stopwatch facility */
double Stopwatch()
{
	double	d;
	static long lasttime=0;
	struct tbuffer {
		long utime;
		long stime;
		long cutime;
		long cstime;
	} buff;
	d=pop();
	if (d == 0.0) {
		lasttime = times((long *)&buff);
		return(0.0);
	}
	else
		return((times((long *)&buff) - lasttime)/60.0);
}

/* execute system call function */
double System()
{
	char	*s;
	int	ret;
	s=spop();
	if (filecount > 0) closedbfile();
	ret = system(s);
	if (filecount > 0) opendbfile(cfilename,0);
	Free(s);
	if (ret == 0)
		return(0.0);
	else {
		sprintf(lasterr,"system() call returns code %d",ret);
		return(ERRVAL);
	}
}

/* annotation functions */

/* primitive annotation match function */
int annmatch(s,start,first,last)
char	*s;		/* specification string */
int	start;		/* annotation number to start search */
int	*first,*last;	/* returned match indexes */
{
	int	i,j,count=0,flag;
	char	t[MAXSTRING],*u,v[MAXSTRING];
	char	*re[MAXRE];

	/* append final separator to input */
	strcpy(t,s);
	strcat(t,ANNSEPS);

	/* find components and compile */
	u = strtok(t,ANNSEPS);
	while ((count<MAXRE) && (u != NULL) && (u[0] != '\0')) {
		strcpy(v,"^");
		strcat(v,u);
		re[count] = regcmp(v);
		if (re[count] == (char *) 0)
			error("illegal regular expression: %s",s);
		count++;
		u = strtok(0,ANNSEPS);
	}

	/* run through list of annotation matches to find sequence */
	*first = -1;
	*last  = -1;
	for (i=start;i<=anncnt-count;i++) {
		if ((anntab[i].time >= timerange[tr][0]) &&
		    (anntab[i].time <  timerange[tr][1]) &&
		    (regex(re[0],anntab[i].ann) != 0)) {
			flag=1;
			for (j=1;j<count;j++)
				if ((anntab[i+j].time >= timerange[tr][1]) ||
				    (regex(re[j],anntab[i+j].ann) == 0))
					flag=0;
			if (flag) {
				*first=i;
				*last=i+count-1;
				break;
			}
		}
	}
	for (i=0;i<count;i++) Free(re[i]);
	if (*first >= 0)
		return(0);
	else
		return(1);
}

/* functions to return time of an annotation specification */
double Time()
{
	char	*s;		/* spec string */
	int	first,last;
	s=spop();
	if (s==ERRSTRING) {
		Free(s);
		return(ERRVAL);
	}
	if (annmatch(s,0,&first,&last) == 0) {
		Free(s);
		return(anntab[first].time);
	}
	Free(s);
	return(ERRVAL);
}
double Timen()
{
	char	*s;		/* spec string */
	double	n;		/* n'th match required */
	int	i,first,last,annloc=0;
	n=pop();
	s=spop();
	if ((n==ERRVAL) || (s==ERRSTRING)) {
		Free(s);
		return(ERRVAL);
	}
	/* skip over first matches */
	for (i=1;i<n;i++) {
		if (annmatch(s,annloc,&first,&last) != 0) return(ERRVAL);
		annloc = last+1;
	}
	/* find required match */
	if (annmatch(s,annloc,&first,&last) == 0) {
		Free(s);
		return(anntab[first].time);
	}
	Free(s);
	return(ERRVAL);
}

/* functions to return length of an annotation specification */
double Length()
{
	char	*s;		/* spec string */
	int	first,last;
	s=spop();
	if (s==ERRSTRING) {
		Free(s);
		return(ERRVAL);
	}
	if (annmatch(s,0,&first,&last) == 0) {
		Free(s);
		return(anntab[last].time - anntab[first].time + anntab[last].length);
	}
	Free(s);
	return(ERRVAL);
}
double Lengthn()
{
	char	*s;		/* spec string */
	double	n;		/* n'th match required */
	int	i,first,last,annloc=0;
	n=pop();
	s=spop();
	if ((n==ERRVAL) || (s==ERRSTRING)) {
		Free(s);
		return(ERRVAL);
	}
	/* skip over first matches */
	for (i=1;i<n;i++) {
		if (annmatch(s,annloc,&first,&last) != 0) return(ERRVAL);
		annloc = last+1;
	}
	/* find required match */
	if (annmatch(s,annloc,&first,&last) == 0) {
		Free(s);
		return(anntab[last].time - anntab[first].time + anntab[last].length);
	}
	Free(s);
	return(ERRVAL);
}

/* function to return text of annotation match */
char *Match()
{
	char	*s;
	int	i,first,last;
	static char matchstr[MAXSTRING];
	s=spop();
	if (s==ERRSTRING) {
		Free(s);
		return(ERRSTRING);
	}
	/* find annotation match */
	if (annmatch(s,0,&first,&last) == 0) {
		Free(s);
		strcpy(matchstr,"");
		for (i=first;i<=last;i++) {
			if ((strlen(matchstr)+strlen(anntab[i].ann)) >= MAXSTRING) {
				runtimewarning("Matched string truncated","");
				return(matchstr);
			}
			else
				strcat(matchstr,anntab[i].ann);
		}
		return(matchstr);
	}
	Free(s);
	return(ERRSTRING);
}

/* function to return text of annotation match */
char *Matchn()
{
	char	*s;
	double	n;
	int	i,first,last,annloc=0;
	static char matchstr[MAXSTRING];
	n=pop();
	s=spop();
	if ((n==ERRVAL) || (s==ERRSTRING)) {
		Free(s);
		return(ERRSTRING);
	}
	/* skip over first matches */
	for (i=1;i<n;i++) {
		if (annmatch(s,annloc,&first,&last) != 0) return(ERRSTRING);
		annloc = last+1;
	}
	/* find relevant annotation match */
	if (annmatch(s,annloc,&first,&last) == 0) {
		Free(s);
		strcpy(matchstr,"");
		for (i=first;i<=last;i++) {
			if ((strlen(matchstr)+strlen(anntab[i].ann)) >= MAXSTRING) {
				runtimewarning("Matched string truncated","");
				return(matchstr);
			}
			else
				strcat(matchstr,anntab[i].ann);
		}
		return(matchstr);
	}
	Free(s);
	return(ERRSTRING);
}

/* function to return number of matches to annotation specification */
double Numberof()
{
	char	*s;
	double	count=0.0;
	int	annloc=0;
	int	first,last;
	s=spop();
	if (s==ERRSTRING) {
		Free(s);
		return(ERRVAL);
	}
	while (annmatch(s,annloc,&first,&last) == 0) {
		annloc = last+1;
		count += 1.0;
	}
	Free(s);
	return(count);
}

/* function to return index of match into string */
extern char *__loc1;
double Index()
{
	char	*r,*s;
	int	i,j,count=0;
	char	t[MAXSTRING],*u;
	char	*re[MAXRE];
	double	posn=0.0;

	s=spop();
	r=spop();
	if ((r==ERRSTRING) || (s==ERRSTRING)) {
		Free(s);
		Free(r);
		return(ERRVAL);
	}
	/* append final separator to input */
	strcpy(t,r);
	strcat(t,ANNSEPS);
	Free(r);

	/* find components and compile */
	u = strtok(t,ANNSEPS);
	while ((count<MAXRE) && (u != NULL) && (u[0] != '\0')) {
		re[count] = (char *)regcmp(u);
		if (re[count] == (char *) 0)
			error("illegal regular expression: %s",s);
		count++;
		u = strtok(0,ANNSEPS);
	}

	/* check each regular expression in turn */
	for (i=0;i<count;i++) {
		if (regex(re[i],s) != 0) {
			posn = __loc1 - s + 1;
			for (j=0;j<count;j++) Free(re[j]);
			Free(s);
			return(posn);
		}
	}
	for (j=0;j<count;j++) Free(re[j]);
	Free(s);
	return(ERRVAL);
}

/* function to compare two strings */
double Compare()
{
	char	*s,*t;
	int	i;
	t=spop();
	s=spop();
	if ((s==ERRSTRING) || (t==ERRSTRING)) {
		Free(s);
		Free(t);
		return(ERRVAL);
	}
	i=strcmp(s,t);
	Free(s);
	Free(t);
	return((double)i);
}

/* function to return index of string in an array of strings */
double Entry()
{
	char	*s,**a;
	int	l,h,n;
	int	i;
	n=(int)vpop();
	h=(int)vpop();
	l=(int)vpop();
	a=(char **)vpop();
	s=spop();
	if (s==ERRSTRING) {
		Free(s);
		return(ERRVAL);
	}
	for (i=0;i<h-l+1;i++) {
		if (s && a[i] && (strcmp(s,a[i]) == 0)) {
			Free(s);
			return((double)(i+l));
		}
	}
	Free(s);
	return(ERRVAL);
}

/* record refreshing of item */
static int stale[MAXDATATYPE];

/* select an item to measure */
double Select()
{
	double	spec;		/* item specification */
	int	it,ty;
	char	itemno[8];

	/* get item type and subtype */
	spec=pop();
	if (!itdecode(spec,&it,&ty)) {
		strcpy(lasterr,"illegal item specification");
		return(ERRVAL);
	}
	/* request load of that type */
	sprintf(itemno,"%d",ty);
	if (ty != 0) ittype[it]=itemno;
	if (loadit(it)) {
		sprintf(lasterr,"could not load item type %d",it);
		return(ERRVAL);
	}
	stale[it]=0;
	/* return # frames */
	return((double)dbitem[it].item.numframes);
}

/* select and return selected item number */
double Selectitem()
{
	double	spec;		/* item specification */
	int	it,ty;
	char	itemno[8];

	/* get item type and subtype */
	spec=pop();
	if (!itdecode(spec,&it,&ty)) {
		strcpy(lasterr,"illegal item specification");
		return(ERRVAL);
	}
	/* request load of that type */
	sprintf(itemno,"%d",ty);
	if (ty != 0) ittype[it]=itemno;
	if (loadit(it)) {
		sprintf(lasterr,"could not load item type %d",it);
		return(ERRVAL);
	}
	stale[it]=0;
	/* return item number */
	if (dbitem[it].item.subtype < 100)
		return(dbitem[it].item.datatype + dbitem[it].item.subtype/100.0);
	else
		return(dbitem[it].item.datatype + dbitem[it].item.subtype/1000.0);
}

/* select an item to measure - next item in file */
double Selectnext()
{
	double	spec;		/* item specification */
	int	it,ty;

	/* get item type and subtype */
	spec=pop();
	if (!itdecode(spec,&it,&ty)) {
		strcpy(lasterr,"illegal item specification");
		return(ERRVAL);
	}
	/* load next item */
	if (loadnextit(it,ty)) {
		sprintf(lasterr,"could not load next item",it);
		return(ERRVAL);
	}
	stale[it]=0;
	/* encode item number */
	if (!itcode(it,dbitem[it].item.subtype,&spec)) return(ERRVAL);
	return(spec);
}

/* select an item to measure by history match */
double Selectmatch()
{
	char	*s;		/* item specification */
	int32	it;
	char	*ty;
	double	spec;

	s=spop();
	if (s==ERRSTRING) {
		Free(s);
		return(ERRVAL);
	}
	if (itspec(s,&it,&ty) != 0) {
		Free(s);
		strcpy(lasterr,"illegal item specification");
		return(ERRVAL);
	}
	ittype[it]=ty;
	if (loadit(it)) {
		Free(s);
		sprintf(lasterr,"could not load item type %d",it);
		return(ERRVAL);
	}
	stale[it]=0;
	Free(s);
	/* encode item number */
	if (!itcode(it,dbitem[it].item.subtype,&spec)) return(ERRVAL);
	return(spec);
}

/* record last used position in Next function */
static int	nextlastposn[MAXDATATYPE];
static double	nextlasttime[MAXDATATYPE];
static double	nextlastsum[MAXDATATYPE];

/* select next time interval in an item */
#define NEXTOFFSET 1.0E-6
double Next()
{
	double	spec;		/* item specification */
	double	t;		/* time */
	double	sum=0.0;
	int	i,pos=0;
	int	it;
	struct lp_rec	*lprec;
	struct co_rec	*corec;
	struct fm_rec	*fmrec;
	struct vu_rec	*vurec;

	t = pop();
	spec=pop();
	if ((spec==ERRVAL) || (t==ERRVAL)) return(ERRVAL);
	it = (int)spec;
	if ((it < 0) || (it > MAXDATATYPE)) {
		runtimewarning("illegal item in 'next' function","");
		return(ERRVAL);
	}
	else {
		if (!dbitem[it].load) {
			if (loadit(it)) return(ERRVAL);
			stale[it]=0;
		}
	}

	/* given a time, find next record */
	if ((it==SP_TYPE) || (it==LX_TYPE) || (it==FX_TYPE) || (it==SY_TYPE) || (it==TR_TYPE)) {
		pos=(int)((t-dbitem[it].item.offset)/dbitem[it].item.frameduration);
		pos++;
		if (pos < 0)
			return(dbitem[it].item.offset+NEXTOFFSET);
		else if (pos >= dbitem[it].item.numframes)
			return(ERRVAL);
		else
			return(dbitem[it].item.offset+NEXTOFFSET+pos*dbitem[it].item.frameduration);
	}
	else if (it==TX_TYPE) {
		pos=(int)((t-dbitem[TX_TYPE].item.offset)/dbitem[TX_TYPE].item.frameduration);
		if (stale[TX_TYPE] && (t==nextlasttime[TX_TYPE])) {
			i = nextlastposn[TX_TYPE];
			sum = nextlastsum[TX_TYPE];
		}
		else {
			i = 0;
			sum = 0.0;
		}
		for (;i<dbitem[TX_TYPE].item.numframes;i++) {
			sum += dbitem[TX_TYPE].u.tx[i];
			if (pos < sum) {
				nextlastposn[TX_TYPE]=i+1;
				nextlastsum[TX_TYPE]=sum;
				stale[TX_TYPE]=1;
				return(nextlasttime[TX_TYPE]=dbitem[TX_TYPE].item.offset+0.000001+sum*dbitem[TX_TYPE].item.frameduration);
			}
		}
	}
	else if (it==AN_TYPE) {
		for (i=0;i<anncnt;i++) {
			if (t < anntab[i].time) return(NEXTOFFSET+anntab[i].time);
		}
	}
	else if (it==LP_TYPE) {
		pos=(int)((t-dbitem[it].item.offset)/dbitem[it].item.frameduration);
		lprec = dbitem[it].u.lp;
		for (i=0;i<dbitem[it].item.numframes;i++) {
			sum += lprec->length2;
			if (pos < sum) return(dbitem[it].item.offset+NEXTOFFSET+sum*dbitem[it].item.frameduration);
			lprec++;
		}
	}
	else if ((it==CO_TYPE) || (it==PC_TYPE)) {
		pos=(int)((t-dbitem[it].item.offset)/dbitem[it].item.frameduration);
		for (i=0;i<dbitem[it].item.numframes;i++) {
			corec = &dbitem[it].u.co[i];
			if (pos < corec->posn) return(dbitem[it].item.offset+NEXTOFFSET+corec->posn*dbitem[it].item.frameduration);
		}
	}
	else if (it==FM_TYPE) {
		pos=(int)((t-dbitem[it].item.offset)/dbitem[it].item.frameduration);
		for (i=0;i<dbitem[it].item.numframes;i++) {
			fmrec = &dbitem[it].u.fm[i];
			if (pos < fmrec->posn) return(dbitem[it].item.offset+NEXTOFFSET+fmrec->posn*dbitem[it].item.frameduration);
		}
	}
	else if (it==VU_TYPE) {
		pos=(int)((t-dbitem[it].item.offset)/dbitem[it].item.frameduration);
		for (i=0;i<dbitem[it].item.numframes;i++) {
			vurec = &dbitem[it].u.vu[i];
			if (pos < vurec->posn) return(dbitem[it].item.offset+NEXTOFFSET+vurec->posn*dbitem[it].item.frameduration);
		}
	}
	return(ERRVAL);
}

/* get item history */
char	*History()
{
	double	spec;
	int	it;

	spec=pop();
	if (spec==ERRVAL) return(ERRSTRING);
	it = (int)spec;
	if ((it < 1) || (it > MAXDATATYPE)) {
		runtimewarning("illegal item in history function","");
		return(ERRSTRING);
	}
	else {
		if (!dbitem[it].load && loadit(it)) return(ERRSTRING);
		return(dbitem[it].item.history);
	}
}

/* get Tx value */
double Tx()
{
	double	t;	/* time */
	int	i,pos,sum=0;
	double	txval;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	/* load tx item */
	if (!dbitem[TX_TYPE].load && loadit(TX_TYPE)) return(ERRVAL);
	pos=(int)((t-dbitem[TX_TYPE].item.offset)/dbitem[TX_TYPE].item.frameduration);
	for (i=0;i<dbitem[TX_TYPE].item.numframes;i++) {
		sum += dbitem[TX_TYPE].u.tx[i];
		if (pos < sum) {
			txval = dbitem[TX_TYPE].u.tx[i] * dbitem[TX_TYPE].item.frameduration;
			return(txval);
		}
	}
	return(ERRVAL);
}

/* get Fx value */
double Fx()
{
	double	t;	/* time */
	int	pos;
	double	fxval;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	/* load fx item */
	if (!dbitem[FX_TYPE].load && loadit(FX_TYPE)) return(ERRVAL);
	pos=(int)((t-dbitem[FX_TYPE].item.offset)/dbitem[FX_TYPE].item.frameduration);
	if ((pos < 0) || (pos >= dbitem[FX_TYPE].item.numframes))
		return(ERRVAL);
	fxval = dbitem[FX_TYPE].u.fx[pos];
	return(fxval);
}

/* get Speech value */
double Sp()
{
	double	t;	/* time */
	int	pos;
	double	spval;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	/* load sp item */
	if (!dbitem[SP_TYPE].load && loadit(SP_TYPE)) return(ERRVAL);
	pos=(int)((t-dbitem[SP_TYPE].item.offset)/dbitem[SP_TYPE].item.frameduration);
	if ((pos < 0) || (pos >= dbitem[SP_TYPE].item.numframes))
		return(ERRVAL);
	spval = dbitem[SP_TYPE].u.sp[pos];
	return(spval);
}

/* get Track value */
double Tr()
{
	double	t;	/* time */
	int	pos;
	double	trval;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	/* load tr item */
	if (!dbitem[TR_TYPE].load && loadit(TR_TYPE)) return(ERRVAL);
	pos=(int)((t-dbitem[TR_TYPE].item.offset)/dbitem[TR_TYPE].item.frameduration);
	if ((pos < 0) || (pos >= dbitem[TR_TYPE].item.numframes))
		return(ERRVAL);
	trval = dbitem[TR_TYPE].u.tr[pos];
	return(trval);
}

/* get Synth Control value */
double  Syval(p,t)
double	p;	/* parameter, 0=FX,3=F1,4=A1,5=B1,6=F2,..,18=V */
double	t;	/* time */
{
	int	pos,offset;
	double	val;
	/* load sy item */
	if (!dbitem[SY_TYPE].load && loadit(SY_TYPE)) return(ERRVAL);
	/* get frame # */
	pos=(int)((t-dbitem[SY_TYPE].item.offset)/dbitem[SY_TYPE].item.frameduration);
	if ((pos < 0) || (pos >= dbitem[SY_TYPE].item.numframes))
		return(ERRVAL);
	/* check offset into frame */
	offset = (int)p;
	if ((offset < 0) || (offset >= dbitem[SY_TYPE].item.framesize))
		return(ERRVAL);
	val = dbitem[SY_TYPE].u.sy[dbitem[SY_TYPE].item.framesize*pos+offset];
	if (val < 0)
		return(ERRVAL);
	else
		return(val);
}

/* SY item access */
double Sy()
{
	double	p,t;
	t=pop();
	p=pop();
	if ((t==ERRVAL) || (p==ERRVAL)) return(ERRVAL);
	return(Syval(p,t));
}

/* F1 value */
double F1()
{
	double t;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	return(Syval(3.0,t));
}

/* A1 value */
double A1()
{
	double	t;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	return(Syval(4.0,t) / 10.0);
}

/* B1 value */
double B1()
{
	double	t;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	return(Syval(5.0,t));
}

/* F2 value */
double F2()
{
	double	t;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	return(Syval(6.0,t));
}

/* A2 value */
double A2()
{
	double	t;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	return(Syval(7.0,t) / 10.0);
}

/* B2 value */
double B2()
{
	double	t;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	return(Syval(8.0,t));
}

/* F3 value */
double F3()
{
	double	t;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	return(Syval(9.0,t));
}

/* A3 value */
double A3()
{
	double	t;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	return(Syval(10.0,t) / 10.0);
}

/* B3 value */
double B3()
{
	double	t;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	return(Syval(11.0,t));
}

/* F4 value */
double F4()
{
	double	t;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	return(Syval(12.0,t));
}

/* A4 value */
double A4()
{
	double	t;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	return(Syval(13.0,t) / 10.0);
}

/* B4 value */
double B4()
{
	double	t;
	t=pop();
	if (t==ERRVAL) return(ERRVAL);
	return(Syval(14.0,t));
}

/* get attribute value from LP item */
double Lp()
{
	char	*aname;
	double	t;
	int		i,j,k,a,pos;
	struct lp_rec	*rec;
	double		totdur;
	char		cname[12];

	t=pop();
	aname=spop();
	if ((t==ERRVAL) || (aname==ERRSTRING)) return(ERRVAL);

	/* check dictionary loaded */
	if (loadlpdict()) return(ERRVAL);

	/* load lp item */
	if (!dbitem[LP_TYPE].load && loadit(LP_TYPE)) return(ERRVAL);

	/* find appropriate entry */
	i=0;
	rec = dbitem[LP_TYPE].u.lp;
	pos = (int)((t - dbitem[LP_TYPE].item.offset)/dbitem[LP_TYPE].item.frameduration);
	if (pos < 0) return(ERRVAL);
	totdur = rec->length2;
	while ((i<dbitem[LP_TYPE].item.numframes) && (pos > totdur)) {
		rec++;
		totdur += rec->length2;
		i++;
	}
	if (i >= dbitem[LP_TYPE].item.numframes) return(ERRVAL);

	/* convert attribute name to compiled form */
	for (j=0;j<12;j++) cname[j]='\0';
	i=0;
	j=0;
	while ((i<strlen(aname)) && (j<12) && (aname[i]!='['))
		cname[j++] = aname[i++];
	if (aname[i] == '[') {
		i++;
		j=0;
		while ((i<strlen(aname)) && (aname[i]!=']')) {
			for (k=11;k>j;k--) cname[k] = cname[k-1];
			cname[j++]=aname[i++];
		}
		if (aname[i] != ']') return(ERRVAL);
		for (k=11;k>j;k--) cname[k] = cname[k-1];
		cname[j] = '|';
	}
	Free(aname);

	/* find attribute index */
	a=0;
	while ((a<lpdictlen) && (strncmp(lpdict[a].aname,cname,8) != 0)) a++;
	if (a >= lpdictlen) return(ERRVAL);

	return((double) rec->alist[a] * lpdict[a].A + lpdict[a].B);
}

/* return energy from coefficients item frame */
double Energy()
{
	double		f,t;
	int		i,ncoeff;
	double		totdur;
	struct co_rec	*rec;
	double		minf,maxf,param();

	t=pop();	/* 2nd argument = time */
	f=pop();	/* 1st argument = frequency */
	if ((t==ERRVAL) || (f==ERRVAL)) return(ERRVAL);

	/* load co item */
	if (!dbitem[CO_TYPE].load && loadit(CO_TYPE)) return(ERRVAL);

	/* find appropriate entry */
	i=0;
	rec = dbitem[CO_TYPE].u.co;
	t = (t - dbitem[CO_TYPE].item.offset);
	if (t < 0) return(ERRVAL);
	totdur = (rec->posn + rec->size)*dbitem[CO_TYPE].item.frameduration;
	while ((i<dbitem[CO_TYPE].item.numframes) && (t > totdur)) {
		rec = &dbitem[CO_TYPE].u.co[i];
		totdur = (rec->posn + rec->size)*dbitem[CO_TYPE].item.frameduration;
		i++;
	}
	if (t > totdur) return(ERRVAL);

	/* find appropriate entry in vector */
	minf = param(dbitem[CO_TYPE].item.params,"minf",0.0);
	maxf = param(dbitem[CO_TYPE].item.params,"maxf",(0.5/dbitem[CO_TYPE].item.frameduration));

	ncoeff = dbitem[CO_TYPE].item.framesize-5;
	i = (int)(ncoeff*(f - minf)/(maxf-minf));
	if ((i<0) || (i>=ncoeff))
		return(ERRVAL);
	else
		return((double) rec->data[i]);

}

/* return value from FM item frame */
double	Fm()
{
	double		p,t;
	int		i,j;
	double		totdur;
	struct fm_rec	*rec;
	float		*fp;

	t=pop();	/* 2nd argument = time */
	p=pop();	/* 1st argument = parameter */
	if ((t==ERRVAL) || (p==ERRVAL)) return(ERRVAL);

	/* load fm item */
	if (!dbitem[FM_TYPE].load && loadit(FM_TYPE)) return(ERRVAL);

	/* find appropriate entry */
	i=0;
	rec = dbitem[FM_TYPE].u.fm;
	t = (t - dbitem[FM_TYPE].item.offset);
	if (t < 0) return(ERRVAL);
	totdur = (rec->posn + rec->size)*dbitem[FM_TYPE].item.frameduration;
	while ((i<dbitem[FM_TYPE].item.numframes) && (t > totdur)) {
		rec = &dbitem[FM_TYPE].u.fm[i];
		totdur = (rec->posn + rec->size)*dbitem[FM_TYPE].item.frameduration;
		i++;
	}
	if (t > totdur) return(ERRVAL);

	if ((p < 0) || (p >= (5+3*rec->npeaks))) return(ERRVAL);
	j = (int)p;
	switch (j) {
	case 0:	return((double)rec->posn);
	case 1: return((double)rec->size);
	case 2: return((double)rec->flag);
	case 3: return((double)rec->gain);
	case 4: return((double)rec->npeaks);
	default:fp = (float *) rec->formant;
		return((double) fp[j-5]);
		break;
	}
}

/* return value from VU item frame */
double	Vu()
{
	double		p,t;
	int		i,j;
	double		totdur;
	struct vu_rec	*rec;

	t=pop();	/* 2nd argument = time */
	p=pop();	/* 1st argument = parameter */
	if ((t==ERRVAL) || (p==ERRVAL)) return(ERRVAL);

	/* load vu item */
	if (!dbitem[VU_TYPE].load && loadit(VU_TYPE)) return(ERRVAL);

	/* find appropriate entry */
	i=0;
	rec = dbitem[VU_TYPE].u.vu;
	t = (t - dbitem[VU_TYPE].item.offset);
	if (t < 0) return(ERRVAL);
	totdur = (rec->posn + rec->size)*dbitem[VU_TYPE].item.frameduration;
	while ((i<dbitem[VU_TYPE].item.numframes) && (t > totdur)) {
		rec = &dbitem[VU_TYPE].u.vu[i];
		totdur = (rec->posn + rec->size)*dbitem[VU_TYPE].item.frameduration;
		i++;
	}
	if (t > totdur) return(ERRVAL);

	if ((p < 0) || (p > 4)) return(ERRVAL);
	j = (int)p;
	switch (j) {
	case 0:	return((double)rec->posn);
	case 1: return((double)rec->size);
	case 2: return((double)rec->flag);
	case 3: return((double)rec->mix);
	case 4: return((double)rec->gain);
	}
	return(ERRVAL);
}

/* return value from CO item frame */
double	Co()
{
	double		p,t;
	int		i,j,ncoeff;
	double		totdur;
	struct pc_rec	*rec;

	t=pop();	/* 2nd argument = time */
	p=pop();	/* 1st argument = parameter */
	if ((t==ERRVAL) || (p==ERRVAL)) return(ERRVAL);

	/* load pc item */
	if (!dbitem[CO_TYPE].load && loadit(CO_TYPE)) return(ERRVAL);

	/* find appropriate entry */
	i=0;
	rec = dbitem[CO_TYPE].u.pc;
	t = (t - dbitem[CO_TYPE].item.offset);
	if (t < 0) return(ERRVAL);
	totdur = (rec->posn + rec->size)*dbitem[CO_TYPE].item.frameduration;
	while ((i<dbitem[CO_TYPE].item.numframes) && (t > totdur)) {
		rec = &dbitem[CO_TYPE].u.pc[i];
		totdur = (rec->posn + rec->size)*dbitem[CO_TYPE].item.frameduration;
		i++;
	}
	if (t > totdur) return(ERRVAL);

	ncoeff = dbitem[CO_TYPE].item.framesize;
	if ((p < 0) || (p >= ncoeff)) return(ERRVAL);
	j = (int)p;
	switch (j) {
	case 0:	return((double)rec->posn);
	case 1: return((double)rec->size);
	case 2: return((double)rec->flag);
	case 3: return((double)rec->mix);
	case 4: return((double)rec->gain);
	default: return((double) rec->data[j-5]);
		break;
	}
}

/* return value from PC item frame */
double	Pc()
{
	double		p,t;
	int		i,j,ncoeff;
	double		totdur;
	struct pc_rec	*rec;

	t=pop();	/* 2nd argument = time */
	p=pop();	/* 1st argument = parameter */
	if ((t==ERRVAL) || (p==ERRVAL)) return(ERRVAL);

	/* load pc item */
	if (!dbitem[PC_TYPE].load && loadit(PC_TYPE)) return(ERRVAL);

	/* find appropriate entry */
	i=0;
	rec = dbitem[PC_TYPE].u.pc;
	t = (t - dbitem[PC_TYPE].item.offset);
	if (t < 0) return(ERRVAL);
	totdur = (rec->posn + rec->size)*dbitem[PC_TYPE].item.frameduration;
	while ((i<dbitem[PC_TYPE].item.numframes) && (t > totdur)) {
		rec = &dbitem[PC_TYPE].u.pc[i];
		totdur = (rec->posn + rec->size)*dbitem[PC_TYPE].item.frameduration;
		i++;
	}
	if (t > totdur) return(ERRVAL);

	ncoeff = dbitem[PC_TYPE].item.framesize;
	if ((p < 0) || (p >= ncoeff)) return(ERRVAL);
	j = (int)p;
	switch (j) {
	case 0:	return((double)rec->posn);
	case 1: return((double)rec->size);
	case 2: return((double)rec->flag);
	case 3: return((double)rec->mix);
	case 4: return((double)rec->gain);
	default: return((double) rec->data[j-5]);
		break;
	}
}

/* string-number conversions */
double Val()
{
	char	*s;
	double	d,atof();
	int	i=0,neg;
	s=spop();
	if (s==ERRSTRING) {
		Free(s);
		return(ERRVAL);
	}
	while (isspace(s[i])) i++;
	if (s[i]=='-') { neg=-1; i++; } else neg=1;
	if ((s[i] < '0') || (s[i] > '9')) {
		Free(s);
		return(ERRVAL);
	}
	else {
		d=atof(s+i);
		Free(s);
		return(d*neg);
	}
}
double Ascii()
{
	char	*s;
	double	d;

	s=spop();
	if (s==ERRSTRING) {
		Free(s);
		return(ERRVAL);
	}
	d = (double)*s;
	Free(s);
	return(d);
}
char  *Istr()
{
	double	v;
	int	i;
	static char	s[MAXSTRING];
	v=pop();
	if (v==ERRVAL) return(ERRSTRING);
	i = (int)v;
	sprintf(s,"%d",i);
	return(s);
}
char  *Str()
{
	double	v,w,d;
	int	iw,id;
	static char	s[MAXSTRING];
	d=pop();
	w=pop();
	v=pop();
	if ((d==ERRVAL) || (w==ERRVAL) || (v==ERRVAL)) return(ERRSTRING);
	iw = (int)w;
	id = (int)d;
	sprintf(s,"%*.*f",iw,id,v);
	return(s);
}
char  *Char()
{
	double	v;
	static char	s[2];
	v=pop();
	if ((v==ERRVAL) || (v < 0) || (v > 255)) return(ERRSTRING);
	s[0] = (char)(int)v;
	s[1] = '\0';
	return(s);
}
double Strlen()
{
	char	*s;
	double	d;
	s=spop();
	if (s==ERRSTRING) {
		Free(s);
		return(ERRVAL);
	}
	d=strlen(s);
	Free(s);
	return(d);
}

/* file functions */
/* open a file for input */
double Openin()
{
	Symbol		*sym;
	char		*name;
	name=spop();
	sym=fpop();
	if (name==ERRSTRING) {
		Free(name);
		return(ERRVAL);
	}
	if (name[strlen(name)-1] == '|') {
		name[strlen(name)-1]='\0';
		if ((sym->u.fil=popen(name,"r")) == NULL) {
			runtimewarning("cannot open pipe for input:",name);
			sym->vtype = BADFIL;
			Free(name);
			return(ERRVAL);
		}
		else {
			sym->vtype = INPIPE;
			Free(name);
			return(0);
		}
	}
	else if ((sym->u.fil=fopen(name,"r")) == NULL) {
		runtimewarning("cannot open for input:",name);
		sym->vtype = BADFIL;
		Free(name);
		return(ERRVAL);
	}
	else {
		sym->vtype = INPFIL;
		Free(name);
		return(0);
	}
}
/* open a file for output */
double Openout()
{
	Symbol		*sym;
	char		*name;
	name=spop();
	sym=fpop();
	if (name==ERRSTRING) {
		Free(name);
		return(ERRVAL);
	}
	if (name[0] == '|') {
		strcpy(name,name+1);
		if ((sym->u.fil=popen(name,"w")) == NULL) {
			runtimewarning("cannot open pipe for output:",name);
			sym->vtype = BADFIL;
			Free(name);
			return(ERRVAL);
		}
		else {
			sym->vtype = OUTPIPE;
			Free(name);
			return(0);
		}
	}
	else if ((sym->u.fil=fopen(name,"w")) == NULL) {
		runtimewarning("cannot open for output:",name);
		sym->vtype = BADFIL;
		Free(name);
		return(ERRVAL);
	}
	else {
		sym->vtype = OUTFIL;
		Free(name);
		return(0);
	}
}
/* open a file for appending */
double Openappend()
{
	Symbol		*sym;
	char		*name;
	name=spop();
	sym=fpop();
	if (name==ERRSTRING) {
		Free(name);
		return(ERRVAL);
	}
	if ((sym->u.fil=fopen(name,"a")) == NULL) {
		runtimewarning("cannot open for appending:",name);
		sym->vtype = BADFIL;
		Free(name);
		return(ERRVAL);
	}
	else {
		sym->vtype = OUTFIL;
		Free(name);
		return(0);
	}
}
/* close a file */
double Close()
{
	Symbol		*sym;
	sym=fpop();
	if ((sym->vtype==INPIPE) || (sym->vtype==OUTPIPE)) {
		if (pclose(sym->u.fil) != 0) {
			sym->vtype = BADFIL;
			return(ERRVAL);
		}
		else {
			sym->vtype = BADFIL;
			return(0.0);
		}
	}
	else if (fclose(sym->u.fil) != 0) {
		sym->vtype = BADFIL;
		return(ERRVAL);
	}
	else {
		sym->vtype = BADFIL;
		return(0.0);
	}
}

/* item functions */
/* load in an existing item */
double SFSGetItem()
{
	Symbol		*sym;
	char		*name;
	char		*arg;
	int			it;
	char		*ty;
	DBITEM		*dbitem;
	int			fid;

	arg=spop();
	name=spop();
	sym=fpop();
	if (arg==ERRSTRING) {
		Free(arg);
		return(ERRVAL);
	}
	if (name==ERRSTRING) {
		Free(name);
		return(ERRVAL);
	}
	if (itspec(arg,&it,&ty) != 0) {
		Free(arg);
		Free(name);
		return(ERRVAL);
	}
	dbitem = (DBITEM *)malloc(sizeof(DBITEM));

	/* open file */
	if ((fid=sfsopen(name,"r",NULL)) < 0) {
		sprintf(lasterr,"could not open file '%s'",name);
		Free(arg);
		Free(name);
		return(ERRVAL);
	}

	/* find item */
	if (!sfsitem(fid,it,ty,&dbitem->item)) {
		sprintf(lasterr,"could not find item in file '%s'",name);
		Free(arg);
		Free(name);
		return(ERRVAL);
	}

	/* get buffer */
	dbitem->item.datapresent=2;		/* force allocation of max sized buffer */
	if ((dbitem->u.ptr=(char *)sfsbuffer(&dbitem->item,dbitem->item.numframes))==NULL) {
		sprintf(lasterr,"could not get buffer for item in file '%s'",name);
		Free(arg);
		Free(name);
		return(ERRVAL);
	}
	dbitem->item.datapresent=0;

	/* read data */
	if (sfsread(fid,0,dbitem->item.numframes,dbitem->u.ptr) != dbitem->item.numframes) {
		sprintf(lasterr,"could not load item from file '%s'",name);
		Free(arg);
		Free(name);
		return(ERRVAL);
	}

	sfsclose(fid);

	sym->u.item = dbitem;
	sym->vtype = INITEM;
	return(0);
}

/* duplicate an existing item */
double SFSDuplicateItem()
{
	Symbol		*isym;
	Symbol		*osym;
	DBITEM		*dbitem;
	int			i,fsize;

	isym=fpop();
	osym=fpop();
	if ((isym->vtype!=INITEM)&&(isym->vtype!=OUTITEM))
		return(ERRVAL);

	dbitem = (DBITEM *)malloc(sizeof(DBITEM));
	memcpy((char *)dbitem,isym->u.item,sizeof(DBITEM));

	/* get buffer */
	dbitem->item.datapresent=2;		/* force allocation of max sized buffer */
	if ((dbitem->u.ptr=(char *)sfsbuffer(&dbitem->item,dbitem->item.numframes))==NULL) {
		sprintf(lasterr,"could not get buffer for item");
		return(ERRVAL);
	}
	dbitem->item.datapresent=0;

	fsize = dbitem->item.framesize*dbitem->item.datasize-sfsstruct[dbitem->item.datatype];
	/* copy data */
	switch (dbitem->item.datatype) {
	case AN_TYPE:
		for (i=0;i<dbitem->item.numframes;i++) {
			dbitem->u.an[i].posn = isym->u.item->u.an[i].posn;
			dbitem->u.an[i].size = isym->u.item->u.an[i].size;
			strcpy(dbitem->u.an[i].label,isym->u.item->u.an[i].label);
		}
		break;
	case DI_TYPE:
		for (i=0;i<dbitem->item.numframes;i++) {
			dbitem->u.di[i].posn = isym->u.item->u.di[i].posn;
			dbitem->u.di[i].size = isym->u.item->u.di[i].size;
			memcpy(dbitem->u.di[i].pixel,isym->u.item->u.di[i].pixel,fsize);
		}
		break;
	case VU_TYPE:
		for (i=0;i<dbitem->item.numframes;i++) {
			dbitem->u.vu[i].posn = isym->u.item->u.vu[i].posn;
			dbitem->u.vu[i].size = isym->u.item->u.vu[i].size;
			dbitem->u.vu[i].flag = isym->u.item->u.vu[i].flag;
			dbitem->u.vu[i].mix = isym->u.item->u.vu[i].mix;
			dbitem->u.vu[i].gain = isym->u.item->u.vu[i].gain;
			memcpy(dbitem->u.vu[i].data,isym->u.item->u.vu[i].data,fsize);
		}
		break;
	case CO_TYPE:
		for (i=0;i<dbitem->item.numframes;i++) {
			dbitem->u.co[i].posn = isym->u.item->u.co[i].posn;
			dbitem->u.co[i].size = isym->u.item->u.co[i].size;
			dbitem->u.co[i].flag = isym->u.item->u.co[i].flag;
			dbitem->u.co[i].mix = isym->u.item->u.co[i].mix;
			dbitem->u.co[i].gain = isym->u.item->u.co[i].gain;
			memcpy(dbitem->u.co[i].data,isym->u.item->u.co[i].data,fsize);
		}
		break;
	case FM_TYPE:
		for (i=0;i<dbitem->item.numframes;i++) {
			dbitem->u.fm[i].posn = isym->u.item->u.fm[i].posn;
			dbitem->u.fm[i].size = isym->u.item->u.fm[i].size;
			dbitem->u.fm[i].flag = isym->u.item->u.fm[i].flag;
			dbitem->u.fm[i].gain = isym->u.item->u.fm[i].gain;
			dbitem->u.fm[i].npeaks = isym->u.item->u.fm[i].npeaks;
			memcpy(dbitem->u.fm[i].formant,isym->u.item->u.fm[i].formant,fsize);
		}
		break;

	default:
		/* printf("copying %d bytes\n",dbitem->item.length); */
		memcpy(dbitem->u.ptr,isym->u.item->u.ptr,dbitem->item.length);
		break;
	}

	/* add new history */
	switch (dbitem->item.datatype) {
	case SP_TYPE:
		sprintf(dbitem->item.history,"sml/SP(script=%s)",sfilename);
		break;
	case LX_TYPE:
		sprintf(dbitem->item.history,"sml/LX(script=%s)",sfilename);
		break;
	case TX_TYPE:
		sprintf(dbitem->item.history,"sml/TX(script=%s)",sfilename);
		break;
	case FX_TYPE:
		sprintf(dbitem->item.history,"sml/FX(script=%s)",sfilename);
		break;
	case AN_TYPE:
		sprintf(dbitem->item.history,"sml/AN(script=%s)",sfilename);
		break;
	case SY_TYPE:
		sprintf(dbitem->item.history,"sml/SY(script=%s)",sfilename);
		break;
	case DI_TYPE:
		sprintf(dbitem->item.history,"sml/DI(script=%s)",sfilename);
		strcpy(dbitem->item.params,"sfsformat");
		break;
	case VU_TYPE:
		sprintf(dbitem->item.history,"sml/VU(script=%s)",sfilename);
		break;
	case CO_TYPE:
		sprintf(dbitem->item.history,"sml/CO(script=%s)",sfilename);
		break;
	case FM_TYPE:
		sprintf(dbitem->item.history,"sml/FM(script=%s)",sfilename);
		break;
	case TR_TYPE:
		sprintf(dbitem->item.history,"sml/TR(script=%s)",sfilename);
		break;
	default:
		return(ERRVAL);
	}

	/* return as new item */
	osym->u.item = dbitem;
	osym->vtype = OUTITEM;
	return(0);
}

/* create an item */
double SFSNewItem()
{
	Symbol		*sym;
	int			itype;
	double		offs;
	double		fdur;
	int			fsize;
	int			numf;

	DBITEM		*dbitem;

	numf=(int)pop();		/* number of frames */
	if ((numf < 1)||(numf>10000000)) return(ERRVAL);
	fsize=(int)pop();		/* frame size */
	if ((fsize < -1)||(fsize>1000)) return(ERRVAL);
	offs=pop();		/* offset */
	if ((offs < -10)||(offs > 10)) return(ERRVAL);
	fdur=pop();		/* frame duration */
	if ((fdur <= 0)||(fdur > 1000)) return(ERRVAL);
	itype=(int)pop();		/* item type */
	if ((itype < SP_TYPE)||(itype > TR_TYPE)) return(ERRVAL);
	sym=fpop();				/* item variable */

	dbitem = (DBITEM *)calloc(1,sizeof(DBITEM));

	/* make the header */
	switch (itype) {
	case SP_TYPE:
		sfsheader(&dbitem->item,SP_TYPE,0,2,1,fdur,offs,1,0,0);
		sprintf(dbitem->item.history,"sml/SP(script=%s)",sfilename);
		break;
	case LX_TYPE:
		sfsheader(&dbitem->item,LX_TYPE,0,2,1,fdur,offs,1,0,0);
		sprintf(dbitem->item.history,"sml/LX(script=%s)",sfilename);
		break;
	case TX_TYPE:
		sfsheader(&dbitem->item,TX_TYPE,0,4,1,fdur,offs,1,0,0);
		sprintf(dbitem->item.history,"sml/TX(script=%s)",sfilename);
		break;
	case FX_TYPE:
		sfsheader(&dbitem->item,FX_TYPE,0,2,1,fdur,offs,1,0,0);
		sprintf(dbitem->item.history,"sml/FX(script=%s)",sfilename);
		break;
	case AN_TYPE:
		sfsheader(&dbitem->item,AN_TYPE,-1,1,-1,fdur,offs,0,0,1);
		sprintf(dbitem->item.history,"sml/AN(script=%s)",sfilename);
		break;
	case SY_TYPE:
		sfsheader(&dbitem->item,SY_TYPE,0,2,fsize,fdur,offs,1,0,0);
		sprintf(dbitem->item.history,"sml/SY(script=%s)",sfilename);
		break;
	case DI_TYPE:
		sfsheader(&dbitem->item,DI_TYPE,-1,1,sfsstruct[DI_TYPE]+fsize,fdur,offs,0,0,1);
		sprintf(dbitem->item.history,"sml/DI(script=%s)",sfilename);
		strcpy(dbitem->item.params,"sfsformat");
		break;
	case VU_TYPE:
		sfsheader(&dbitem->item,VU_TYPE,-1,4,sfsstruct[VU_TYPE]/4+fsize,fdur,offs,0,0,1);
		sprintf(dbitem->item.history,"sml/VU(script=%s)",sfilename);
		break;
	case CO_TYPE:
		sfsheader(&dbitem->item,CO_TYPE,-1,4,sfsstruct[CO_TYPE]/4+fsize,fdur,offs,0,0,1);
		sprintf(dbitem->item.history,"sml/CO(script=%s)",sfilename);
		break;
	case FM_TYPE:
		sfsheader(&dbitem->item,FM_TYPE,-1,4,sfsstruct[FM_TYPE]/4+fsize,fdur,offs,0,0,1);
		sprintf(dbitem->item.history,"sml/FM(script=%s)",sfilename);
		break;
	case TR_TYPE:
		sfsheader(&dbitem->item,TR_TYPE,1,4,1,fdur,offs,1,0,0);
		sprintf(dbitem->item.history,"sml/TR(script=%s)",sfilename);
		break;
	default:
		return(ERRVAL);
	}

	/* get a buffer */
	if ((dbitem->u.ptr=(char *)sfsbuffer(&dbitem->item,numf))==NULL) {
		sprintf(lasterr,"could not get buffer of size %d frames\n",numf);
		return(ERRVAL);
	}

	dbitem->item.numframes=numf;
	sym->u.item = dbitem;
	sym->vtype = OUTITEM;
	return(0);

}

/* store an item */
double SFSPutItem()
{
	Symbol		*sym;
	char		*name;
	int			numf;
	int			fid;

	numf=(int)pop();	/* number of frames */
	name=spop();	/* filename */
	sym=fpop();		/* item */
	if (numf <= 0) {
		Free(name);
		return(ERRVAL);
	}
	if (name==ERRSTRING) {
		Free(name);
		return(ERRVAL);
	}
	if (sym->vtype!=OUTITEM) {
		sprintf(lasterr,"cannot save input item directly");
		return(ERRVAL);
	}

	/* create output channel */
	if ((fid=sfschannel(name,&sym->u.item->item)) < 0) {
		sprintf(lasterr,"could not open output channel to '%s'",name);
		Free(name);
		return(ERRVAL);
	}

	/* write data to channel */
	if (sfswrite(fid,numf,sym->u.item->u.ptr) != numf) {
		sprintf(lasterr,"write error on '%s'",name);
		Free(name);
		return(ERRVAL);
	}

	/* update data file */
	if (!sfsupdate(name)) {
		sprintf(lasterr,"update error on '%s'",name);
		Free(name);
		return(ERRVAL);
	}

	return(numf);
}

/* get an item header field */
double SFSGetItemHeaderField()
{
	Symbol		*sym;
	char		*name;

	name=spop();
	sym=fpop();
	if (name==ERRSTRING) {
		Free(name);
		return(ERRVAL);
	}
	if ((sym->vtype!=INITEM)&&(sym->vtype!=OUTITEM))
		return(ERRVAL);
	if (strcmp(name,"numframes")==0) {
		return((double)sym->u.item->item.numframes);
	}
	else if (strcmp(name,"frameduration")==0) {
		return((double)sym->u.item->item.frameduration);
	}
	else if (strcmp(name,"offset")==0) {
		return((double)sym->u.item->item.offset);
	}
	else if (strcmp(name,"framesize")==0) {
		if (sym->u.item->item.datatype==AN_TYPE)
			return(256);
		else
			return((double)sym->u.item->item.framesize-sfsstruct[sym->u.item->item.datatype]/sym->u.item->item.datasize);
	}
	else if (strcmp(name,"itemno")==0) {
		return((double)(sym->u.item->item.datatype+sym->u.item->item.subtype/100.0));
	}
	else
		return(ERRVAL);
}

/* get an item header string field */
char * SFSGetItemHeaderString()
{
	Symbol		*sym;
	char		*name;
	static char	tbuf[32];

	name=spop();
	sym=fpop();
	if (name==ERRSTRING) {
		Free(name);
		return(ERRSTRING);
	}
	if ((sym->vtype!=INITEM)&&(sym->vtype!=OUTITEM))
		return(ERRSTRING);
	if (strcmp(name,"history")==0) {
		return(sym->u.item->item.history);
	}
	else if (strcmp(name,"params")==0) {
		return(sym->u.item->item.params);
	}
	else if (strcmp(name,"processdate")==0) {
		strcpy(tbuf,ctime(&(sym->u.item->item.processdate)));
		tbuf[24]='\0';
		return(tbuf);
	}
	else if (strcmp(name,"itemno")==0) {
		sprintf(tbuf,"%d.%02d",sym->u.item->item.datatype,sym->u.item->item.subtype);
		return(tbuf);
	}
	else
		return(ERRSTRING);
}

/* get a value from a frame */
double SFSGetItemData()
{
	int	idx;
	int	fno;
	Symbol	*sym;

	idx=(int)pop();
	fno=(int)pop();
	sym=fpop();
	if ((sym->vtype!=INITEM)&&(sym->vtype!=OUTITEM))
		return(ERRVAL);
	if ((fno < 0)||(fno >= sym->u.item->item.numframes))
		return(ERRVAL);
	if ((idx < 0)||(idx >= sym->u.item->item.framesize-sfsstruct[sym->u.item->item.datatype]/sym->u.item->item.datasize))
		return(ERRVAL);
	switch (sym->u.item->item.datatype) {
	case SP_TYPE:
		return((double)sym->u.item->u.sp[fno]);
	case LX_TYPE:
		return((double)sym->u.item->u.lx[fno]);
	case TX_TYPE:
		return((double)sym->u.item->u.tx[fno]);
	case FX_TYPE:
		return((double)sym->u.item->u.fx[fno]);
	case SY_TYPE:
		return((double)sym->u.item->u.sy[fno*sym->u.item->item.framesize+idx]);
	case TR_TYPE:
		return((double)sym->u.item->u.tr[fno]);
	case DI_TYPE:
		return((double)sym->u.item->u.di[fno].pixel[idx]);
	case VU_TYPE:
		return((double)sym->u.item->u.vu[fno].data[idx]);
	case CO_TYPE:
		return((double)sym->u.item->u.co[fno].data[idx]);
	case FM_TYPE:
		switch (idx%3) {
		case 0:
			return((double)sym->u.item->u.fm[fno].formant[idx/3].freq);
		case 1:
			return((double)sym->u.item->u.fm[fno].formant[idx/3].amp);
		case 2:
			return((double)sym->u.item->u.fm[fno].formant[idx/3].band);
		}
	}
	return(ERRVAL);
}

/* get array of values from item */
double SFSGetItemArray()
{
	int	lo,hi,ncol;
	double	*arr;
	int	cnt;
	int	fno;
	Symbol	*sym;
	int	i;

	ncol = (int)vpop();
	hi = (int)vpop();
	lo = (int)vpop();
	arr = (double *)vpop();
	cnt=(int)pop();
	fno=(int)pop();
	sym=fpop();
	if ((sym->vtype!=INITEM)&&(sym->vtype!=OUTITEM))
		return(ERRVAL);
	if ((fno < 0)||(fno >= sym->u.item->item.numframes))
		return(ERRVAL);

	if (cnt > (hi-lo+1)) cnt=(hi-lo+1);
	if ((fno+cnt) > sym->u.item->item.numframes) cnt = sym->u.item->item.numframes - fno;
	switch (sym->u.item->item.datatype) {
	case SP_TYPE:
		for (i=0;i<cnt;i++)
			arr[i]=(double)sym->u.item->u.sp[fno+i];
		break;
	case LX_TYPE:
		for (i=0;i<cnt;i++)
			arr[i]=(double)sym->u.item->u.lx[fno+i];
		break;
	case TX_TYPE:
		for (i=0;i<cnt;i++)
			arr[i]=(double)sym->u.item->u.tx[fno+i];
		break;
	case FX_TYPE:
		for (i=0;i<cnt;i++)
			arr[i]=(double)sym->u.item->u.fx[fno+i];
		break;
	case TR_TYPE:
		for (i=0;i<cnt;i++)
			arr[i]=(double)sym->u.item->u.tr[fno+i];
		break;
	default:
		return(ERRVAL);
	}
	return(cnt);
}


/* get a field from a frame */
double SFSGetItemField()
{
	int	idx;
	int	fno;
	Symbol	*sym;

	idx=(int)pop();
	fno=(int)pop();
	sym=fpop();
	if ((sym->vtype!=INITEM)&&(sym->vtype!=OUTITEM))
		return(ERRVAL);
	if ((fno < 0)||(fno >= sym->u.item->item.numframes))
		return(ERRVAL);
	if (sym->u.item->item.datatype!=AN_TYPE) {
		if ((idx < 0)||(idx >= sym->u.item->item.framesize))
			return(ERRVAL);
	}
	switch (sym->u.item->item.datatype) {
	case AN_TYPE:
		switch (idx) {
		case 0:
			return((double)sym->u.item->u.an[fno].posn);
		case 1:
			return((double)sym->u.item->u.an[fno].size);
		}
		break;
	case DI_TYPE:
		switch (idx) {
		case 0:
			return((double)sym->u.item->u.di[fno].posn);
		case 1:
			return((double)sym->u.item->u.di[fno].size);
		}
		break;
	case VU_TYPE:
		switch (idx) {
		case 0:
			return((double)sym->u.item->u.vu[fno].posn);
		case 1:
			return((double)sym->u.item->u.vu[fno].size);
		case 2:
			return((double)sym->u.item->u.vu[fno].flag);
		case 3:
			return((double)sym->u.item->u.vu[fno].mix);
		case 4:
			return((double)sym->u.item->u.vu[fno].gain);
		}
		break;
	case CO_TYPE:
		switch (idx) {
		case 0:
			return((double)sym->u.item->u.co[fno].posn);
		case 1:
			return((double)sym->u.item->u.co[fno].size);
		case 2:
			return((double)sym->u.item->u.co[fno].flag);
		case 3:
			return((double)sym->u.item->u.co[fno].mix);
		case 4:
			return((double)sym->u.item->u.co[fno].gain);
		}
		break;
	case FM_TYPE:
		switch (idx) {
		case 0:
			return((double)sym->u.item->u.fm[fno].posn);
		case 1:
			return((double)sym->u.item->u.fm[fno].size);
		case 2:
			return((double)sym->u.item->u.fm[fno].flag);
		case 3:
			return((double)sym->u.item->u.fm[fno].gain);
		case 4:
			return((double)sym->u.item->u.fm[fno].npeaks);
		}
		break;
	}
	return(ERRVAL);
}

/* get a field from a frame */
char * SFSGetItemString()
{
	int	idx;
	int	fno;
	Symbol	*sym;

	fno=(int)pop();
	sym=fpop();
	if ((sym->vtype!=INITEM)&&(sym->vtype!=OUTITEM))
		return(ERRSTRING);
	if ((fno < 0)||(fno >= sym->u.item->item.numframes))
		return(ERRSTRING);
	switch (sym->u.item->item.datatype) {
	case AN_TYPE:
		return(sym->u.item->u.an[fno].label);
	}
	return(ERRSTRING);
}

/* set an item header string field */
double SFSSetItemHeaderString()
{
	Symbol		*sym;
	char		*name;
	char		*value;

	value=spop();
	name=spop();
	sym=fpop();
	if (value==ERRSTRING) {
		Free(value);
		return(ERRVAL);
	}
	if (name==ERRSTRING) {
		Free(name);
		return(ERRVAL);
	}
	if (sym->vtype!=OUTITEM)
		return(ERRVAL);
	if (strcmp(name,"history")==0) {
		strncpy(sym->u.item->item.history,value,sizeof(sym->u.item->item.history)-1);
		return(0.0);
	}
	else if (strcmp(name,"params")==0) {
		strncpy(sym->u.item->item.params,value,sizeof(sym->u.item->item.params)-1);
		return(0.0);
	}
	else
		return(ERRVAL);
}

/* set a value into frame */
double SFSSetItemData()
{
	double val;
	int	idx;
	int	fno;
	Symbol	*sym;

	val=pop();
	idx=(int)pop();
	fno=(int)pop();
	sym=fpop();
	if (sym->vtype!=OUTITEM)
		return(ERRVAL);
	if ((fno < 0)||(fno >= sym->u.item->item.numframes))
		return(ERRVAL);
	if ((idx < 0)||(idx >= sym->u.item->item.framesize-sfsstruct[sym->u.item->item.datatype]/sym->u.item->item.datasize))
		return(ERRVAL);
	switch (sym->u.item->item.datatype) {
	case SP_TYPE:
		sym->u.item->u.sp[fno]=(short)val;
		break;
	case LX_TYPE:
		sym->u.item->u.lx[fno]=(short)val;
		break;
	case TX_TYPE:
		sym->u.item->u.tx[fno]=(int)val;
		break;
	case FX_TYPE:
		sym->u.item->u.fx[fno]=(short)val;
		break;
	case SY_TYPE:
		sym->u.item->u.sy[fno*sym->u.item->item.framesize+idx]=(short)val;
		break;
	case TR_TYPE:
		sym->u.item->u.tr[fno]=(float)val;
		break;
	case DI_TYPE:
		sym->u.item->u.di[fno].pixel[idx]=(unsigned char)(int)val;
		break;
	case VU_TYPE:
		sym->u.item->u.vu[fno].data[idx]=(float)val;
		break;
	case CO_TYPE:
		sym->u.item->u.co[fno].data[idx]=(float)val;
		break;
	case FM_TYPE:
		switch (idx%3) {
		case 0:
			sym->u.item->u.fm[fno].formant[idx/3].freq=(float)val;
			break;
		case 1:
			sym->u.item->u.fm[fno].formant[idx/3].amp=(float)val;
			break;
		case 2:
			sym->u.item->u.fm[fno].formant[idx/3].band=(float)val;
			break;
		}
		break;
	default:
		return(ERRVAL);
	}
	return(val);
}

/* set a field into frame */
double SFSSetItemField()
{
	double val;
	int	idx;
	int	fno;
	Symbol	*sym;

	val=pop();
	idx=(int)pop();
	fno=(int)pop();
	sym=fpop();
	if (sym->vtype!=OUTITEM)
		return(ERRVAL);
	if ((fno < 0)||(fno >= sym->u.item->item.numframes))
		return(ERRVAL);
	if (sym->u.item->item.datatype!=AN_TYPE) {
		if ((idx < 0)||(idx >= sym->u.item->item.framesize))
			return(ERRVAL);
	}
	switch (sym->u.item->item.datatype) {
	case AN_TYPE:
		switch (idx) {
		case 0:
			return(sym->u.item->u.an[fno].posn=(int)val);
		case 1:
			return(sym->u.item->u.an[fno].size=(int)val);
		}
		break;
	case DI_TYPE:
		switch (idx) {
		case 0:
			return(sym->u.item->u.di[fno].posn=(int)val);
		case 1:
			return(sym->u.item->u.di[fno].size=(int)val);
		}
		break;
	case VU_TYPE:
		switch (idx) {
		case 0:
			return(sym->u.item->u.vu[fno].posn=(int)val);
		case 1:
			return(sym->u.item->u.vu[fno].size=(int)val);
		case 2:
			return(sym->u.item->u.vu[fno].flag=(int)val);
		case 3:
			return(sym->u.item->u.vu[fno].mix=(float)val);
		case 4:
			return(sym->u.item->u.vu[fno].gain=(float)val);
		}
		break;
	case CO_TYPE:
		switch (idx) {
		case 0:
			return(sym->u.item->u.co[fno].posn=(int)val);
		case 1:
			return(sym->u.item->u.co[fno].size=(int)val);
		case 2:
			return(sym->u.item->u.co[fno].flag=(int)val);
		case 3:
			return(sym->u.item->u.co[fno].mix=(float)val);
		case 4:
			return(sym->u.item->u.co[fno].gain=(float)val);
		}
		break;
	case FM_TYPE:
		switch (idx) {
		case 0:
			return(sym->u.item->u.fm[fno].posn=(int)val);
		case 1:
			return(sym->u.item->u.fm[fno].size=(int)val);
		case 2:
			return(sym->u.item->u.fm[fno].flag=(int)val);
		case 3:
			return(sym->u.item->u.fm[fno].gain=(float)val);
		case 4:
			return(sym->u.item->u.fm[fno].npeaks=(int)val);
		}
		break;
	}
	return(ERRVAL);
}

/* set a string into frame */
double SFSSetItemString()
{
	char	*str;
	int	fno;
	Symbol	*sym;

	str=spop();
	fno=(int)pop();
	sym=fpop();
	if (sym->vtype!=OUTITEM)
		return(ERRVAL);
	if ((fno < 0)||(fno >= sym->u.item->item.numframes))
		return(ERRVAL);
	switch (sym->u.item->item.datatype) {
	case AN_TYPE:
		strncpy(sym->u.item->u.an[fno].label,str,256);
		break;
	default:
		return(ERRVAL);
	}
	return(0);
}

static char * gentempfilename()
{
	static char tfilename[256];
#if defined(DOS)||defined(WIN32)
	char	tdir[256];
	GetTempPath(256,tdir);
	GetTempFileName(tdir,"sml",0,tfilename);
#else
	strcpy(tfilename,"/tmp/sml.XXXXXX");
	mktemp(tfilename);
#endif
	return(tfilename);
}

/* process an item through an SFS program */
double SFSProcessItem()
{
	Symbol		*isym;
	Symbol		*osym;
	char		*name;
	char		*arg;
	char		fname[256];
	int			fid;
	int			numf;
	int			it;
	char		*ty;
	DBITEM		*dbitem;
	char		cmd[512];
	int			code;

	arg=spop();		/* output item type */
	isym=fpop();	/* input item */
	name=spop();	/* progname */
	osym=fpop();	/* output item */

	if (arg==ERRSTRING) {
		Free(arg);
		return(ERRVAL);
	}
	if (name==ERRSTRING) {
		Free(arg);
		Free(name);
		return(ERRVAL);
	}
	if ((isym->vtype!=INITEM)&&(isym->vtype!=OUTITEM)) {
		Free(arg);
		Free(name);
		return(ERRVAL);
	}

	/* create temporary SFS file */
	strcpy(fname,gentempfilename());
	fid = sfsopen(fname,"h",NULL);
	if (fid >= 0) sfsclose(fid);

	/* create output channel */
	if ((fid=sfschannel(fname,&isym->u.item->item)) < 0) {
		sprintf(lasterr,"could not open output channel to '%s'",fname);
		Free(arg);
		Free(name);
		return(ERRVAL);
	}

	/* write data to channel */
	numf=isym->u.item->item.numframes;
	if (sfswrite(fid,numf,isym->u.item->u.ptr) != numf) {
		sprintf(lasterr,"write error on '%s'",fname);
		Free(arg);
		Free(name);
		return(ERRVAL);
	}

	/* update data file */
	if (!sfsupdate(fname)) {
		sprintf(lasterr,"update error on '%s'",fname);
		Free(arg);
		Free(name);
		return(ERRVAL);
	}

	/* do process */
	sprintf(cmd,"%s \"%s\"",name,fname);
	if ((code=system(cmd))!=0) {
		sprintf(lasterr,"system() call returns code %d",code);
		Free(arg);
		Free(name);
		return(ERRVAL);
	}

	/* check output item needed */
	if (strcmp(arg,"")==0) {
		Free(arg);
		Free(name);
		return(0);
	}

	/* find last item */
	if (itspec(arg,&it,&ty) != 0) {
		sprintf(lasterr,"illegal item specification");
		Free(arg);
		Free(name);
		return(ERRVAL);
	}
	dbitem = (DBITEM *)malloc(sizeof(DBITEM));

	/* open file */
	if ((fid=sfsopen(fname,"r",NULL)) < 0) {
		sprintf(lasterr,"could not open '%s'",fname);
		Free(arg);
		Free(name);
		return(ERRVAL);
	}

	/* find item */
	if (!sfsitem(fid,it,ty,&dbitem->item)) {
		sprintf(lasterr,"could not find item type %d in '%s'",it,fname);
		Free(arg);
		Free(name);
		return(ERRVAL);
	}

	/* get buffer */
	if ((dbitem->u.ptr=(char *)sfsbuffer(&dbitem->item,dbitem->item.numframes))==NULL) {
		sprintf(lasterr,"could not get buffer for item in '%s'",fname);
		Free(arg);
		Free(name);
		return(ERRVAL);
	}

	/* read data */
	if (sfsread(fid,0,dbitem->item.numframes,dbitem->u.ptr) != dbitem->item.numframes) {
		sprintf(lasterr,"read error on '%s'",fname);
		Free(arg);
		Free(name);
		return(ERRVAL);
	}

	sfsclose(fid);

	osym->u.item = dbitem;
	osym->vtype = OUTITEM;
	return(0);

}

/* statistics functions */
struct threshold {
	double	freedom;
	double	thresh;
};
struct threshold t01[19]={
{	1,	63.66 },
{	2,	9.92 },
{	3,	5.84 },
{	4,	4.60 },
{	5,	4.03 },
{	6,	3.71 },
{	7,	3.50 },
{	8,	3.36 },
{	9,	3.25 },
{	10,	3.17 },
{	12,	3.05 },
{	15,	2.95 },
{	20,	2.85 },
{	24,	2.80 },
{	30,	2.75 },
{	40,	2.70 },
{	60,	2.66 },
{	120,	2.62 },
{	1E38,	2.58 }
};
double Ttest01(s1,s2)
double s1[],s2[];
{
	double	parent,diff,t,fabs();
	int	i,f;

	if ((s1[COUNT] < 2) || (s2[COUNT] < 2)) return(ERRVAL);
	f = (int)(s1[COUNT] + s2[COUNT] - 2);
	parent = (s1[COUNT]*s1[VARIANCE]+s2[COUNT]*s2[VARIANCE])/f;
	diff = sqrt(parent * (s1[COUNT]+s2[COUNT]) / (s1[COUNT]*s2[COUNT]));
	t = (s1[MEAN]-s2[MEAN])/diff;
	i=0;
	while (f > t01[i].freedom) i++;
	if (fabs(t) > t01[i].thresh)
		return(t);
	else
		return(ERRVAL);
}
double Correl(s1,s2,p)
double s1[],s2[],p[];
{
	double	sxy,sx,sy,r,sqrt();

	if ((s1[COUNT] < 2) || (s1[COUNT] != s2[COUNT]) || (s1[COUNT] != p[COUNT])) return(ERRVAL);

	sxy = p[MEAN] - s1[MEAN]*s2[MEAN];

	sx = sqrt(s1[SUMSQ]/s1[COUNT] - (s1[SUM]*s1[SUM])/(s1[COUNT]*s1[COUNT]));
	sy = sqrt(s2[SUMSQ]/s2[COUNT] - (s2[SUM]*s2[SUM])/(s2[COUNT]*s2[COUNT]));

	r = sxy / (sx * sy);

	return(r);
}
double Anovar(s,l)
double s[];
int    l;
{
	int	i;
	double	totnum=0,totval=0;
	double	dfb,dfw,dft;
	double	sumb=0,sumw=0,sumt=0;
	double	meanb,meanw;
	double	fratio;
	double	*p;

	if (l < 2 ) return(ERRVAL);
	for (i=0;i<l;i++) {
		p = &s[6*i];
		if (p[COUNT] < 2) return(ERRVAL);
		totnum += p[COUNT];
		totval += p[SUM];
		sumb += p[SUM]*p[SUM]/p[COUNT];
		sumt += p[SUMSQ];
	}
	dfb = l-1;
	dft = totnum - 1;
	dfw = dft - dfb;
	sumb -= totval*totval/totnum;
	sumt -= totval*totval/totnum;
	sumw = sumt - sumb;
	meanb = sumb / dfb;
	meanw = sumw / dfw;
	if (meanw == 0) return(ERRVAL);
	fratio = meanb / meanw;
	return(fratio);
}

/* item numbering encoding scheme */
int itcode(it,ty,val)
int	it,ty;
double	*val;
{
	*val=ERRVAL;
	if ((it < 0) || (it > MAXDATATYPE))
		return(0);
	if ((ty < 0) || (ty > 999))
		return(0);
	if (ty < 100)
		*val = (100.0*it + ty)/100.0;
	else
		*val = (100000.0*it + ty)/100000.0;
	return(1);
}
int itdecode(val,it,ty)
double	val;
int	*it,*ty;
{
	*it=0;
	*ty=0;
	if (val==ERRVAL)
		return(0);
	if ((val < 0) || (val > MAXDATATYPE+1))
		return(0);
	*it = (int)val;
	val = val - *it;
	if (val <= 0.00999)
		*ty = (int)(100000.0 * val + 0.5);
	else
		*ty = (int)(100.0 * val + 0.5);
	return(1);
}

#include "fft.h"

/* FFT routine */
double FFTransform(void)
{
	int	lo1,hi1,nc1,sz1;
	int	lo2,hi2,nc2,sz2;
	int	lo3,hi3,nc3,sz3;
	double	*ar1;
	double	*ar2;
	double	*ar3;
	int		size;
	float	*spect;
	int		i,xsize;

	nc3 = (int)vpop();
	hi3 = (int)vpop();
	lo3 = (int)vpop();
	ar3 = (double *)vpop();
	sz3 = hi3-lo3+1;

	nc2 = (int)vpop();
	hi2 = (int)vpop();
	lo2 = (int)vpop();
	ar2 = (double *)vpop();
	sz2 = hi2-lo2+1;

	size = (int)pop();

	nc1 = (int)vpop();
	hi1 = (int)vpop();
	lo1 = (int)vpop();
	ar1 = (double *)vpop();
	sz1 = hi1-lo1+1;

	xsize=16;
	while (xsize < size) xsize *= 2;

	spect = (float *)calloc(xsize,sizeof(float));

	for (i=0;i<size;i++) spect[i] = (float)ar1[i];

	REALFFT(spect,xsize/2,FORWARD);

	for (i=0;(i<xsize/2)&&(i<sz2);i++)
		ar2[i] = sqrt(spect[2*i]*spect[2*i]+spect[2*i+1]*spect[2*i+1]);
	for (i=0;(i<xsize/2)&&(i<sz3);i++)
		ar3[i] = atan2(spect[2*i+1],spect[2*i]);

	free(spect);

	return(xsize/2);
}

/*========================================================================================*/
/* A C-program for MT19937: Real number version                */
/*   genrand() generates one pseudorandom real number (double) */
/* which is uniformly distributed on [0,1]-interval, for each  */
/* call. sgenrand(seed) set initial values to the working area */
/* of 624 words. Before genrand(), sgenrand(seed) must be      */
/* called once. (seed is any 32-bit integer except for 0).     */
/* Integer generator is obtained by modifying two lines.       */
/*   Coded by Takuji Nishimura, considering the suggestions by */
/* Topher Cooper and Marc Rieffel in July-Aug. 1997.           */

/* This library is free software; you can redistribute it and/or   */
/* modify it under the terms of the GNU Library General Public     */
/* License as published by the Free Software Foundation; either    */
/* version 2 of the License, or (at your option) any later         */
/* version.                                                        */
/* This library is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of  */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.            */
/* See the GNU Library General Public License for more details.    */
/* You should have received a copy of the GNU Library General      */
/* Public License along with this library; if not, write to the    */
/* Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA   */
/* 02111-1307  USA                                                 */

/* Copyright (C) 1997 Makoto Matsumoto and Takuji Nishimura.       */
/* Any feedback is very welcome. For any question, comments,       */
/* see http://www.math.keio.ac.jp/matumoto/emt.html or email       */
/* matumoto@math.keio.ac.jp                                        */

/* Period parameters */
#define N 624
#define M 397
#define MATRIX_A 0x9908b0df   /* constant vector a */
#define UPPER_MASK 0x80000000 /* most significant w-r bits */
#define LOWER_MASK 0x7fffffff /* least significant r bits */

/* Tempering parameters */
#define TEMPERING_MASK_B 0x9d2c5680
#define TEMPERING_MASK_C 0xefc60000
#define TEMPERING_SHIFT_U(y)  (y >> 11)
#define TEMPERING_SHIFT_S(y)  (y << 7)
#define TEMPERING_SHIFT_T(y)  (y << 15)
#define TEMPERING_SHIFT_L(y)  (y >> 18)

static unsigned long mt[N]; /* the array for the state vector  */
static int mti=N+1; /* mti==N+1 means mt[N] is not initialized */

/* initializing the array with a NONZERO seed */
static void
sgenrand(seed)
    unsigned long seed;
{
    /* setting initial seeds to mt[N] using         */
    /* the generator Line 25 of Table 1 in          */
    /* [KNUTH 1981, The Art of Computer Programming */
    /*    Vol. 2 (2nd Ed.), pp102]                  */
    mt[0]= seed & 0xffffffff;
    for (mti=1; mti<N; mti++)
        mt[mti] = (69069 * mt[mti-1]) & 0xffffffff;
}

static double /* generating reals */
/* unsigned long */ /* for integer generation */
genrand()
{
    unsigned long y;
    static unsigned long mag01[2]={0x0, MATRIX_A};
    /* mag01[x] = x * MATRIX_A  for x=0,1 */

    if (mti >= N) { /* generate N words at one time */
        int kk;

        if (mti == N+1)   /* if sgenrand() has not been called, */
            sgenrand(4357); /* a default initial seed is used   */

        for (kk=0;kk<N-M;kk++) {
            y = (mt[kk]&UPPER_MASK)|(mt[kk+1]&LOWER_MASK);
            mt[kk] = mt[kk+M] ^ (y >> 1) ^ mag01[y & 0x1];
        }
        for (;kk<N-1;kk++) {
            y = (mt[kk]&UPPER_MASK)|(mt[kk+1]&LOWER_MASK);
            mt[kk] = mt[kk+(M-N)] ^ (y >> 1) ^ mag01[y & 0x1];
        }
        y = (mt[N-1]&UPPER_MASK)|(mt[0]&LOWER_MASK);
        mt[N-1] = mt[M-1] ^ (y >> 1) ^ mag01[y & 0x1];

        mti = 0;
    }

    y = mt[mti++];
    y ^= TEMPERING_SHIFT_U(y);
    y ^= TEMPERING_SHIFT_S(y) & TEMPERING_MASK_B;
    y ^= TEMPERING_SHIFT_T(y) & TEMPERING_MASK_C;
    y ^= TEMPERING_SHIFT_L(y);

    return ( (double)y / (unsigned long)0xffffffff ); /* reals */
    /* return y; */ /* for integer generation */
}

double Srandom(void)
{
	double d=pop();
	unsigned long seed;

	if (d==0.0)
		seed = (unsigned long)time((time_t *)0);
	else
		seed = (unsigned long)d;
	sgenrand(seed);
	return((double)seed);
}

double Random(void)
{
	double d=pop();
	return genrand()*d;
}
